/**
  *  \file t_maybe.cc
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "cpluslib/maybe.h"
#include "cpluslib/types.h"

#include "tests.h"

typedef SimpleMaybe<int16,0x8000> may16;
typedef SimpleMaybe<uint16,0xFFFF> umay16;

void
TestMaybe::test1()
{
    may16 a, b(99), c(20);
    TS_ASSERT_EQUALS(a.getRawValue(), int16(0x8000));
    TS_ASSERT_EQUALS(b, 99);
    TS_ASSERT_EQUALS(c, 20);
    TS_ASSERT(!a.isKnown());
    TS_ASSERT(b.isKnown());
    TS_ASSERT(c.isKnown());
    TS_ASSERT_EQUALS(b*c, 1980);

    a = may16(b+1);
    TS_ASSERT(a.isKnown());
    TS_ASSERT_EQUALS(a, 100);

    umay16 d;
    TS_ASSERT_EQUALS(d.getRawValue(), 0xFFFF);
    // d = umay16(c);        does not compile
    d = umay16(c.getValue());
    TS_ASSERT(d.isKnown());
    TS_ASSERT_EQUALS(d, 20);
    TS_ASSERT_EQUALS(c, 20);
}
